"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nonce = exports.schemaIsArray = exports.transformMessages = exports.getValidatorBag = exports.nested = void 0;
/**
 * Get the typed schema of a nested validator class.
 * @param nestedClass Validator class
 * @returns Typed schema of validator class.
 */
const nested = (nestedClass) => exports.getValidatorBag(nestedClass).schema;
exports.nested = nested;
/**
 * Get the validation bag of a target.
 *
 * If none exists, target is initialized with an empty bag.
 * @param target Target
 * @returns Validation Schema.
 */
const getValidatorBag = (target) => {
    const metadataKey = "@classValidatorBag";
    const prototype = (target && target.prototype) || target;
    const metadata = Reflect.getMetadata(metadataKey, prototype);
    if (metadata)
        return metadata;
    const key = `${exports.nonce()}.${prototype.constructor.name}`;
    const validatorBag = { key, schema: {}, messages: {} };
    Reflect.defineMetadata(metadataKey, validatorBag, prototype);
    return Reflect.getMetadata(metadataKey, prototype);
};
exports.getValidatorBag = getValidatorBag;
/**
 * Converts messages field name into a format compatible with Adonis.
 * @param fieldName Field name
 * @param messages Messages rules for the field specified
 * @returns Transformed messages
 */
const transformMessages = (fieldName, messages, isArray = false) => Object.entries(messages).reduce((prev, [key, value]) => {
    prev[`${fieldName}${isArray ? ".*." : "."}${key}`] = value;
    return prev;
}, {});
exports.transformMessages = transformMessages;
/**
 * Check if a schema is an array schema.
 * @param schema Schema to check.
 * @returns If schema is an array.
 */
const schemaIsArray = (schema) => "getTree" in schema && schema.getTree().type === "array";
exports.schemaIsArray = schemaIsArray;
/**
 * Generate a unique and ever increasing nonce of length 15.
 * @returns Unique number
 */
exports.nonce = ((length = 15) => {
    let [last, repeat] = [Date.now(), 0];
    return () => {
        const now = Math.pow(10, 2) * +new Date();
        if (now === last)
            repeat++;
        else
            [last, repeat] = [now, 0];
        const s = (now + repeat).toString();
        return +s.substr(s.length - length);
    };
})();
